@@ -13,6 +13,7 @@ use salvo_core::http::headers::HeaderName;
 use salvo_core::http::uri::Scheme;
 use salvo_core::http::{Method, StatusCode};
 use salvo_core::prelude::*;
+use salvo_core::routing::FlowCtrl;
 
 /// key used to save csrf data to depot.
 pub const DATA_KEY: &str = "::salvo::extra::csrf::data";
@@ -29,34 +30,34 @@ pub trait CsrfDepotExt {
     /// a query parameter, or a form field.
     fn csrf_token(&self) -> Option<&str>;
 
-    /// Gets the name of the header in which to return the CSRF token,
+    /// Gets the name of the header in which to returns the CSRF token,
     /// if the CSRF token is being returned in a header.
     fn csrf_header_name(&self) -> Option<&str>;
 
-    /// Gets the name of the query param in which to return the CSRF
+    /// Gets the name of the query param in which to returns the CSRF
     /// token, if the CSRF token is being returned in a query param.
     fn csrf_query_param(&self) -> Option<&str>;
 
-    /// Gets the name of the form field in which to return the CSRF
+    /// Gets the name of the form field in which to returns the CSRF
     /// token, if the CSRF token is being returned in a form field.
     fn csrf_field_name(&self) -> Option<&str>;
 }
 
 impl CsrfDepotExt for Depot {
     fn csrf_token(&self) -> Option<&str> {
-        self.try_borrow::<CsrfData>(DATA_KEY).map(|d| &*d.token)
+        self.get::<CsrfData>(DATA_KEY).map(|d| &*d.token)
     }
 
     fn csrf_header_name(&self) -> Option<&str> {
-        self.try_borrow::<CsrfData>(DATA_KEY).map(|d| d.header_name.as_str())
+        self.get::<CsrfData>(DATA_KEY).map(|d| d.header_name.as_str())
     }
 
     fn csrf_query_param(&self) -> Option<&str> {
-        self.try_borrow::<CsrfData>(DATA_KEY).map(|d| &*d.query_param)
+        self.get::<CsrfData>(DATA_KEY).map(|d| &*d.query_param)
     }
 
     fn csrf_field_name(&self) -> Option<&str> {
-        self.try_borrow::<CsrfData>(DATA_KEY).map(|d| &*d.field_name)
+        self.get::<CsrfData>(DATA_KEY).map(|d| &*d.field_name)
     }
 }
 
@@ -226,7 +227,7 @@ impl CsrfHandler {
         // body specifically) if we find a CSRF token in an earlier
         // location. And we can't use `or_else` chaining since the
         // function that searches through the form body is async. Note
-        // that if parsing the body fails then we want to return an
+        // that if parsing the body fails then we want to returns an
         // InternalServerError, hence the `?`. This is not the same as
         // what we will do later, which is convert failures to *parse* a
         // found CSRF token into Forbidden responses.
@@ -264,7 +265,7 @@ impl CsrfHandler {
 
 #[salvo_core::async_trait]
 impl Handler for CsrfHandler {
-    async fn handle(&self, req: &mut Request, depot: &mut Depot, res: &mut Response) {
+    async fn handle(&self, req: &mut Request, depot: &mut Depot, res: &mut Response, ctrl: &mut FlowCtrl) {
         // We always begin by trying to find the existing CSRF cookie,
         // even if we do not need to protect this method. A new token is
         // generated on every request *based on the encrypted key in the
@@ -283,16 +284,19 @@ impl Handler for CsrfHandler {
                     } else {
                         tracing::debug!("rejecting request due to invalid or expired CSRF token");
                         res.set_status_code(StatusCode::FORBIDDEN);
+                        ctrl.skip_reset();
                         return;
                     }
                 } else {
                     tracing::debug!("rejecting request due to missing CSRF token",);
                     res.set_status_code(StatusCode::FORBIDDEN);
+                    ctrl.skip_reset();
                     return;
                 }
             } else {
                 tracing::debug!("rejecting request due to missing CSRF cookie",);
                 res.set_status_code(StatusCode::FORBIDDEN);
+                ctrl.skip_reset();
                 return;
             }
         }
@@ -316,6 +320,7 @@ impl Handler for CsrfHandler {
         // Add the CSRF cookie to the response.
         let cookie = self.build_cookie(secure_cookie, cookie.b64_string());
         res.add_cookie(cookie);
+        ctrl.call_next(req, depot, res).await;
     }
 }
 
@@ -343,7 +348,7 @@ mod tests {
 
     #[tokio::test]
     async fn middleware_exposes_csrf_request_extensions() {
-        let router = Router::new().before(CsrfHandler::new(&SECRET)).get(get_index);
+        let router = Router::new().hoop(CsrfHandler::new(&SECRET)).get(get_index);
         let service = Service::new(router);
 
         let req: Request = hyper::Request::builder()
@@ -358,7 +363,7 @@ mod tests {
 
     #[tokio::test]
     async fn middleware_adds_csrf_cookie_sets_request_token() {
-        let router = Router::new().before(CsrfHandler::new(&SECRET)).get(get_index);
+        let router = Router::new().hoop(CsrfHandler::new(&SECRET)).get(get_index);
         let service = Service::new(router);
 
         let req: Request = hyper::Request::builder()
@@ -377,7 +382,7 @@ mod tests {
     #[tokio::test]
     async fn middleware_validates_token_in_header() {
         let router = Router::new()
-            .before(CsrfHandler::new(&SECRET))
+            .hoop(CsrfHandler::new(&SECRET))
             .get(get_index)
             .post(post_index);
         let service = Service::new(router);
@@ -419,7 +424,7 @@ mod tests {
     #[tokio::test]
     async fn middleware_validates_token_in_alternate_header() {
         let router = Router::new()
-            .before(CsrfHandler::new(&SECRET).with_header_name(HeaderName::from_static("x-mycsrf-header")))
+            .hoop(CsrfHandler::new(&SECRET).with_header_name(HeaderName::from_static("x-mycsrf-header")))
             .get(get_index)
             .post(post_index);
         let service = Service::new(router);
@@ -461,7 +466,7 @@ mod tests {
     #[tokio::test]
     async fn middleware_validates_token_in_query() {
         let router = Router::new()
-            .before(CsrfHandler::new(&SECRET))
+            .hoop(CsrfHandler::new(&SECRET))
             .get(get_index)
             .post(post_index);
         let service = Service::new(router);
@@ -501,7 +506,7 @@ mod tests {
     #[tokio::test]
     async fn middleware_validates_token_in_alternate_query() {
         let router = Router::new()
-            .before(CsrfHandler::new(&SECRET).with_query_param("my-csrf-token"))
+            .hoop(CsrfHandler::new(&SECRET).with_query_param("my-csrf-token"))
             .get(get_index)
             .post(post_index);
         let service = Service::new(router);
@@ -542,7 +547,7 @@ mod tests {
     #[tokio::test]
     async fn middleware_validates_token_in_form() {
         let router = Router::new()
-            .before(CsrfHandler::new(&SECRET).with_query_param("my-csrf-token"))
+            .hoop(CsrfHandler::new(&SECRET).with_query_param("my-csrf-token"))
             .get(get_index)
             .post(post_index);
         let service = Service::new(router);
@@ -583,7 +588,7 @@ mod tests {
     #[tokio::test]
     async fn middleware_validates_token_in_alternate_form() {
         let router = Router::new()
-            .before(CsrfHandler::new(&SECRET).with_form_field("my-csrf-token"))
+            .hoop(CsrfHandler::new(&SECRET).with_form_field("my-csrf-token"))
             .get(get_index)
             .post(post_index);
         let service = Service::new(router);
@@ -624,7 +629,7 @@ mod tests {
     #[tokio::test]
     async fn middleware_rejects_short_token() {
         let router = Router::new()
-            .before(CsrfHandler::new(&SECRET))
+            .hoop(CsrfHandler::new(&SECRET))
             .get(get_index)
             .post(post_index);
         let service = Service::new(router);
@@ -664,7 +669,7 @@ mod tests {
     #[tokio::test]
     async fn middleware_rejects_invalid_base64_token() {
         let router = Router::new()
-            .before(CsrfHandler::new(&SECRET))
+            .hoop(CsrfHandler::new(&SECRET))
             .get(get_index)
             .post(post_index);
         let service = Service::new(router);
@@ -704,7 +709,7 @@ mod tests {
     #[tokio::test]
     async fn middleware_rejects_mismatched_token() {
         let router = Router::new()
-            .before(CsrfHandler::new(&SECRET))
+            .hoop(CsrfHandler::new(&SECRET))
             .get(get_index)
             .post(post_index);
         let service = Service::new(router);
